'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _slicedToArray = (function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i['return']) _i['return'](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError('Invalid attempt to destructure non-iterable instance'); } }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _postcss = require('postcss');

var _postcss2 = _interopRequireDefault(_postcss);

var _replaceSymbols = require('./replace-symbols');

var _replaceSymbols2 = _interopRequireDefault(_replaceSymbols);

var matchImports = /^(.+?)\s+from\s+("[^"]*"|'[^']*'|[\w-]+)$/;
var matchLet = /(?:,\s+|^)([\w-]+):?\s+("[^"]*"|'[^']*'|[^,]+)\s?/g;
var matchImport = /^([\w-]+)(?:\s+as\s+([\w-]+))?/;
var options = {};
var importIndex = 0;
var createImportedName = options && options.createImportedName || function (importName /*, path*/) {
  return 'i__const_' + importName.replace(/\W/g, '_') + '_' + importIndex++;
};

exports['default'] = function (css) {
  /* Find any local let rules and store them*/
  var translations = {};
  css.eachAtRule(/^-?let$/, function (atRule) {
    var matches = undefined;
    while (matches = matchLet.exec(atRule.params)) {
      var _matches = matches;

      var _matches2 = _slicedToArray(_matches, 3);

      var /*match*/key = _matches2[1];
      var value = _matches2[2];

      translations[key] = value;
      atRule.removeSelf();
    }
  });

  /* We want to export anything defined by now, but don't add it to the CSS yet or
  it well get picked up by the replacement stuff */
  var exportDeclarations = Object.keys(translations).map(function (key) {
    return _postcss2['default'].decl({
      value: translations[key],
      prop: key,
      before: "\n  ",
      _autoprefixerDisabled: true
    });
  });

  /* Find imports and insert ICSS tmp vars */
  var importAliases = [];
  css.eachAtRule(/^-?import$/, function (atRule) {
    var matches = matchImports.exec(atRule.params);
    if (matches) {
      var _matches3 = _slicedToArray(matches, 3);

      var /*match*/aliases = _matches3[1];
      var path = _matches3[2];

      // We can use constants for path names
      if (translations[path]) path = translations[path];
      var imports = aliases.split(/\s*,\s*/).map(function (alias) {
        var tokens = matchImport.exec(alias);
        if (tokens) {
          var _tokens = _slicedToArray(tokens, 3);

          var /*match*/theirName = _tokens[1];
          var _tokens$2 = _tokens[2];
          var myName = _tokens$2 === undefined ? theirName : _tokens$2;

          var importedName = createImportedName(myName);
          translations[myName] = importedName;
          return { theirName: theirName, importedName: importedName };
        } else {
          throw new Error('@import statement "' + alias + '" is invalid!');
        }
      });
      importAliases.push({ path: path, imports: imports });
      atRule.removeSelf();
    }
  });

  /* If we have no translations, don't continue */
  if (!Object.keys(translations).length) return;

  /* Perform replacements */
  (0, _replaceSymbols2['default'])(css, translations);

  /* Add import rules */
  importAliases.forEach(function (_ref) {
    var path = _ref.path;
    var imports = _ref.imports;

    css.prepend(_postcss2['default'].rule({
      selector: ':import(' + path + ')',
      after: "\n",
      nodes: imports.map(function (_ref2) {
        var theirName = _ref2.theirName;
        var importedName = _ref2.importedName;
        return _postcss2['default'].decl({
          value: theirName,
          prop: importedName,
          before: "\n  ",
          _autoprefixerDisabled: true
        });
      })
    }));
  });

  /* Add export rules if any */
  if (exportDeclarations.length > 0) {
    css.prepend(_postcss2['default'].rule({
      selector: ':export',
      after: "\n",
      nodes: exportDeclarations
    }));
  }
};

module.exports = exports['default'];